#version 330

uniform	vec4 diffuse;
uniform float roughness;

in Data {
	vec3 normal;
	vec3 l_dir;
	vec3 pos_cam;
} DataIn;

out vec4 colorOut;

void main() {

	// normalize both input vectors
	vec3 n = normalize(DataIn.normal);
	vec3 l = DataIn.l_dir;
	// compute view vector
	vec3 v = normalize(-DataIn.pos_cam);

	// aux variables to simplify equations
	float roughness_2 = roughness * roughness;

	float dot_NL = max(0, dot(n,l));
	float dot_NV = max(0, dot(n,v));
	float acos_n_l = acos(clamp(dot_NL,-1,1));
	float acos_n_v = acos(clamp(dot_NV,-1,1));

	// project view and light vectors on normal plane
	vec3 proj_l = normalize(l - dot_NL*n);
	vec3 proj_v = normalize(v - dot_NV*n);

	float A = 1 - 0.5 * ( roughness_2 / (roughness_2 + 0.33));
	float B = 0.45 * roughness_2 / (roughness_2 + 0.09);
	float C = max(0,dot(proj_v, proj_l));
	float alpha = max(acos_n_l, acos_n_v);
	float beta = min(acos_n_l, acos_n_v);

	float intensity = dot_NL * (A + B * sin(alpha) * tan(beta) * max(0, C)) ;

	colorOut = max(intensity *  diffuse, diffuse * 0.05);
}
