#version 330

uniform	vec4 diffuse;
uniform vec4 specular;
uniform float alpha_x;
uniform float alpha_y;

in Data {
	vec3 normal;
    vec3 tangent;
    vec3 bitangent;
	vec3 l_dir;
	vec3 pos_cam;
} DataIn;

out vec4 colorOut;

void main() {

	// normalize both input vectors
	vec3 n = normalize(DataIn.normal);
	vec3 l = DataIn.l_dir;
	// compute view vector
	vec3 v = normalize(-DataIn.pos_cam);
    vec3 t = normalize(DataIn.tangent);
    vec3 b = normalize(DataIn.bitangent);

    // half vector
    vec3 h = normalize(l+v);

	// aux variables to simplify equations

	float dot_NL = max(0.00000f, dot(n,l));
	float dot_NV = max(0.00000f, dot(n,v));
    float dot_HT = max(0.00000f, dot(h,t));
    float dot_HB = max(0.00000f, dot(h,b));
    float dot_HN = max(0.00000f, dot(h,n));

    float A = 1.0 / (sqrt(max(0, dot_NL * dot_NV)));
    float B = 1.0 / (4 * 3.14159 * alpha_x * alpha_y);
    float C = pow(dot_HT/alpha_x, 2) + pow(dot_HB/alpha_y, 2);

    float specInt = 0.0;
    float intensity = max(0, dot_NL);

    if (dot_NL > 0.00000f)
        specInt = dot_NL * A * B * exp(-2 * C /(1 + dot_HN));


	colorOut = max(intensity *  diffuse + specInt * specular, diffuse * 0.05); 

    
}
